#include <fcntl.h>
#include <stdio.h>

#include <sys/mman.h>

#include <asm/io.h>
#undef HAE_ADDRESS
#include <asm/apecs.h>	/* need this on non-APECS based system */


#define APECS_BASE	APECS_MEM_GCR
#define APECS_SIZE	(APECS_IOC_TBIA + 8 - APECS_BASE)


static inline unsigned long
rd (unsigned long base, unsigned long reg_addr)
{
    unsigned long *addr = (unsigned long *)(reg_addr - APECS_BASE + base);

    return *addr;
}


void
apecs_info (void)
{
    unsigned long apecs;
    unsigned long v, size;
    int fd;

    fd = open("/dev/mem", O_RDWR);

    if (fd < 0) {
	perror("/dev/mem");
	exit(1);
    }
    apecs = (unsigned long)
	mmap(0, APECS_SIZE, PROT_READ, MAP_SHARED, fd, APECS_BASE);
    if ((long) apecs == -1) {
	perror("mmap");
	exit(1);
    }

    printf(" DC21071-CA (Memory Controller):\n");

    v = rd(apecs, APECS_MEM_GCR);

    printf("  DMA arbitration mode\t: ");
    switch ((v >> 1) & 0x3) {
	case 0x0: printf("CPU priority\n"); break;
	case 0x1: printf("CPU priority\n"); break;
	case 0x2: printf("DMA priority\n"); break;
	case 0x3: printf("DMA strong priority\n"); break;
    }
    printf("  Memory width\t\t: %s bits\n", ((v >> 4) & 1) ? "128" : "64");

    size = rd(apecs, APECS_MEM_TAR) & ~1L;
    size &= -size;
    size = (size << 16) / 1024;
    printf("  B-cache\t\t: %ld KB, %s%s%s\n", size,
	   ((v >> 5) & 1) ? "enabled" : "disabled",
	   ((v >> 6) & 1) ? ", no allocate mode" : "",
	   ((v >> 7) & 1) ? ", slow writes" : "");

    printf(" DC21071-DA (I/O Controller):\n");

    v = rd(apecs, APECS_IOC_DCSR);
    printf("  Pass 2 chip\t\t: %s\n", ((v >> 31) & 1) ? "yes" : "no");
    printf("  Read bypass\t\t: ");
    switch ((v >> 16) & 0x3) {
      case 0x0: printf("full bypass\n"); break;
      case 0x1: printf("illegal (reserved)\n"); break;
      case 0x2: printf("partial bypass\n"); break;
      case 0x3: printf("no bypass\n"); break;
    }
    printf("  Parity error checking\t: %s\n",
	   ((v >> 4) & 1) ? "disabled" : "enabled");
    printf("  Correctable error intr: %s\n",
	   ((v >> 3) & 1) ? "disabled" : "enabled");
    printf("  DMA prefetching\t: %s\n",
	   ((v >> 2) & 1) ? "enabled" : "disabled");
    printf("  I/O TLB\t\t: %s\n",
	   ((v >> 0) & 1) ? "enabled" : "disabled");
    printf("  Master latency timer\t: %d\n",
	   (__u32) rd(apecs, APECS_IOC_PMLT));
}
