#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/mman.h>

#include <asm/io.h>
#undef HAE_ADDRESS
#include <asm/lca.h>	/* need this on non-LCA based system */

#define LCA_BASE	LCA_MEM_BCR0
#define LCA_SIZE	(LCA_IOC_TB_TAG7 + 8 - LCA_BASE)

#define BCR0	 0	/* bank configuration 0 */
#define BCR1	 1	/* bank configuration 1 */
#define BCR2	 2	/* bank configuration 2 */
#define BCR3	 3	/* bank configuration 3 */
#define BMR0	 4	/* bank address mask 0 */
#define BMR1	 5	/* bank address mask 1 */
#define BMR2	 6	/* bank address mask 2 */
#define BMR3	 7	/* bank address mask 3 */
#define BTR0	 8	/* bank timing 0 */
#define BTR1	 9	/* bank timing 1 */
#define BTR2	10	/* bank timing 2 */
#define BTR3	11	/* bank timing 3 */
#define GTR	12	/* global timing */
#define ESR	13	/* error status register */
#define EAR	14	/* error address register */
#define CAR	15	/* cache register */
#define VGR	16	/* video and graphics control */
#define PLM	17	/* plane mask */
#define FOR	18	/* foreground */
#define MEMSIZE	(19*sizeof(long))


static inline unsigned long
rd (unsigned long base, unsigned long reg_addr)
{
    unsigned long *addr = (unsigned long *)(reg_addr - LCA_BASE + base);

    return *addr;
}


void
lca_info (void)
{
    unsigned long lca, v, scale;
    int fd, i;

    fd = open("/dev/mem", O_RDWR);
    if (fd < 0) {
	perror("/dev/mem");
	exit(1);
    }
    lca = (unsigned long)
	mmap(0, LCA_SIZE, PROT_READ, MAP_SHARED, fd, LCA_BASE);
    if ((long) lca == -1) {
	perror("mmap");
	exit(1);
    }

    printf(" LCA Memory Controller:\n");

    v = rd(lca, LCA_MEM_CAR);

    printf("  b-cache\t\t: %d KB, %s, %s%s\n", 1<<(((v >> 5) & 0x7) + 6),
	   ((v >> 0) & 1) ? "enabled" : "disabled",
	   (((v >> 2) & 1) ? "parity checking enabled" :
	    ((v >> 4) & 1) ? "ECC enabled" : "no error checking"),
	   ((v >> 14) & 1) ? ", slow writes" : "");
    printf("  b-cache power saving\t: %s\n", ((v >> 15) & 1) ? "on" : "off");

    printf("  memory bank ECC enable: ");
    for (i = 0; i < 4; ++i) {
	v = rd(lca, LCA_MEM_BCR0 + 8*i);
	printf("%s%s", (i > 0) ? ", " : "", ((v >> 10) & 1) ? "on" : "off");
    }
    printf("\n");

    v = rd(lca, LCA_MEM_GTR);
    scale = ((v >> 27) & 1) ? 512 : 64;
    printf("  memory refresh\t: %s, %ld cycles\n",
	   ((v >> 18) & 1) ? "enabled" : "disabled",
	   ((v >> 19) & 0xff) * scale);
    
}
